from .classes import NeedleApp
from .wrappers import init_wrapper_store
import threading


def init_app(flask_app=None, debug=False, show_blocked_message=False, sdk_version=''):
    global needle_app
    needle_app = NeedleApp(debug=debug, show_blocked_message=show_blocked_message, flask_app=flask_app)
    needle_app.agent_version = sdk_version

    if not needle_app.run_agent:
        print('Needle.sh: Stopping agent.')
        return

    # Initialise wrappers
    init_wrapper_store()

    try:
        # Start thread to send data to Needle.sh server
        x = threading.Thread(target=needle_app.api_thread, args=(), daemon=True)
        x.start()
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting Needle.sh thread to send data', error_data)

    return needle_app


def get_needle_app():
    global needle_app
    return needle_app


needle_app = None


# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _libinjection
else:
    import _libinjection

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def version():
    return _libinjection.version()

def sqli(s, fingerprint):
    return _libinjection.sqli(s, fingerprint)

def xss(s):
    return _libinjection.xss(s)
FLAG_NONE = _libinjection.FLAG_NONE
FLAG_QUOTE_NONE = _libinjection.FLAG_QUOTE_NONE
FLAG_QUOTE_SINGLE = _libinjection.FLAG_QUOTE_SINGLE
FLAG_QUOTE_DOUBLE = _libinjection.FLAG_QUOTE_DOUBLE
FLAG_SQL_ANSI = _libinjection.FLAG_SQL_ANSI
FLAG_SQL_MYSQL = _libinjection.FLAG_SQL_MYSQL
LOOKUP_WORD = _libinjection.LOOKUP_WORD
LOOKUP_TYPE = _libinjection.LOOKUP_TYPE
LOOKUP_OPERATOR = _libinjection.LOOKUP_OPERATOR
LOOKUP_FINGERPRINT = _libinjection.LOOKUP_FINGERPRINT
class sqli_token(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    pos = property(_libinjection.sqli_token_pos_get)
    len = property(_libinjection.sqli_token_len_get)
    count = property(_libinjection.sqli_token_count_get)
    type = property(_libinjection.sqli_token_type_get)
    str_open = property(_libinjection.sqli_token_str_open_get)
    str_close = property(_libinjection.sqli_token_str_close_get)
    val = property(_libinjection.sqli_token_val_get)

    def __init__(self):
        _libinjection.sqli_token_swiginit(self, _libinjection.new_sqli_token())
    __swig_destroy__ = _libinjection.delete_sqli_token

# Register sqli_token in _libinjection:
_libinjection.sqli_token_swigregister(sqli_token)

class sqli_state(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    s = property(_libinjection.sqli_state_s_get)
    slen = property(_libinjection.sqli_state_slen_get)
    lookup = property(_libinjection.sqli_state_lookup_get)
    userdata = property(_libinjection.sqli_state_userdata_get)
    flags = property(_libinjection.sqli_state_flags_get)
    pos = property(_libinjection.sqli_state_pos_get)
    current = property(_libinjection.sqli_state_current_get)
    fingerprint = property(_libinjection.sqli_state_fingerprint_get)
    reason = property(_libinjection.sqli_state_reason_get)
    stats_comment_ddw = property(_libinjection.sqli_state_stats_comment_ddw_get)
    stats_comment_ddx = property(_libinjection.sqli_state_stats_comment_ddx_get)
    stats_comment_c = property(_libinjection.sqli_state_stats_comment_c_get)
    stats_comment_hash = property(_libinjection.sqli_state_stats_comment_hash_get)
    stats_folds = property(_libinjection.sqli_state_stats_folds_get)
    stats_tokens = property(_libinjection.sqli_state_stats_tokens_get)

    def __init__(self):
        _libinjection.sqli_state_swiginit(self, _libinjection.new_sqli_state())
    __swig_destroy__ = _libinjection.delete_sqli_state

# Register sqli_state in _libinjection:
_libinjection.sqli_state_swigregister(sqli_state)


def sqli_get_token(sqlistate, i):
    return _libinjection.sqli_get_token(sqlistate, i)

def sqli_init(sql_state, s, flags):
    return _libinjection.sqli_init(sql_state, s, flags)

def is_sqli(sql_state):
    return _libinjection.is_sqli(sql_state)

def sqli_callback(sql_state, fn):
    return _libinjection.sqli_callback(sql_state, fn)

def sqli_reset(sql_state, flags):
    return _libinjection.sqli_reset(sql_state, flags)

def sqli_fingerprint(sql_state, flags):
    return _libinjection.sqli_fingerprint(sql_state, flags)

def sqli_lookup_word(sql_state, lookup_type, s):
    return _libinjection.sqli_lookup_word(sql_state, lookup_type, s)

def sqli_tokenize(sql_state):
    return _libinjection.sqli_tokenize(sql_state)

def sqli_fold(sql_state):
    return _libinjection.sqli_fold(sql_state)

def sqli_check_fingerprint(sql_state):
    return _libinjection.sqli_check_fingerprint(sql_state)

def sqli_blacklist(sql_state):
    return _libinjection.sqli_blacklist(sql_state)

def sqli_not_whitelist(sql_state):
    return _libinjection.sqli_not_whitelist(sql_state)





from .classes import RequestData
import threading
import time

wrappers = None
needle_data = threading.local()

# Initialise wrappers
def init_wrapper_store():
    global wrappers

    wrappers = []

    try:
        lang_module = 'django.core.handlers.base.BaseHandler.get_response'
        from django.core.handlers.base import BaseHandler
        add_wrapper_record(lang_module, BaseHandler.get_response, needle_django_get_response)
    except ImportError:
        pass

    try:
        lang_module = 'django.template.loader.render_to_string'
        import django.template.loader
        add_wrapper_record(lang_module, django.template.loader.render_to_string, needle_django_template_render)
    except ImportError:
        pass

    try:
        lang_module = 'flask.signals.request_started'
        add_wrapper_record(lang_module, None, needle_flask_request_started)
    except ImportError:
        pass

    try:
        lang_module = 'flask.signals.request_finished'
        add_wrapper_record(lang_module, None, needle_flask_request_finished)
    except ImportError:
        pass

    try:
        lang_module = 'flask.render_template'
        import flask
        add_wrapper_record(lang_module, flask.render_template, needle_flask_render_template)
    except ImportError:
        pass

    try:
        lang_module = 'mysql.connector.connect'
        import mysql.connector
        add_wrapper_record(lang_module, mysql.connector.connect, needle_mysql_connect)
    except ImportError:
        pass

    try:
        lang_module = 'psycopg2.connect'
        import psycopg2
        add_wrapper_record(lang_module, psycopg2.connect, needle_psycopg2_connect)
    except ImportError:
        pass

    try:
        lang_module = 'os.system'
        import os
        add_wrapper_record(lang_module, os.system, needle_os_system)
    except ImportError:
        pass

    try:
        lang_module = 'os.popen'
        import os
        add_wrapper_record(lang_module, os.popen, needle_os_popen)
    except ImportError:
        pass

    try:
        lang_module = 'builtins.open'
        import builtins
        add_wrapper_record(lang_module, builtins.open, needle_builtins_open)
    except ImportError:
        pass

    try:
        lang_module = 'requests.get'
        import requests
        add_wrapper_record(lang_module, requests.get, needle_requests_get)
    except ImportError:
        pass

    try:
        lang_module = 'urllib.request.urlopen'
        import urllib.request
        add_wrapper_record(lang_module, urllib.request.urlopen, needle_urllib_request_urlopen)
    except ImportError:
        pass


# Add wrapper record
def add_wrapper_record(lang_module, orig, wrapper):
    global wrappers
    wrappers.append({'lang_module': lang_module, 'orig': orig, 'wrapper': wrapper})


# Get wrapper
def get_wrapper(lang_module):
    global wrappers
    wrapper = None

    for w in wrappers:
        if w['lang_module'] == lang_module:
            wrapper = w['wrapper']
            break

    return wrapper


# Get original
def get_orig(lang_module):
    global wrappers
    orig = None

    for w in wrappers:
        if w['lang_module'] == lang_module:
            orig = w['orig']
            break

    return orig


# Set wrapper
def set_wrapper(lang_module, set_flag):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    try:
        if set_flag:
            func = get_wrapper(lang_module)
        else:
            func = get_orig(lang_module)

        if lang_module == 'django.core.handlers.base.BaseHandler.get_response':
            try:
                from django.core.handlers.base import BaseHandler
                BaseHandler.get_response = func
            except ImportError:
                pass

        if lang_module == 'django.template.loader.render_to_string':
            try:
                import django.template.loader
                django.template.loader.render_to_string = func
            except ImportError:
                pass

        if lang_module == 'flask.render_template':
            try:
                import flask
                flask.render_template = func
            except ImportError:
                pass

        if lang_module == 'mysql.connector.connect':
            try:
                import mysql.connector
                mysql.connector.connect = func
            except ImportError:
                pass

        if lang_module == 'psycopg2.connect':
            try:
                import psycopg2
                psycopg2.connect = func
            except ImportError:
                pass

        if lang_module == 'os.system':
            try:
                import os
                os.system = func
            except ImportError:
                pass

        if lang_module == 'os.popen':
            try:
                import os
                os.popen = func
            except ImportError:
                pass

        if lang_module == 'builtins.open':
            try:
                import builtins
                builtins.open = func
            except ImportError:
                pass

        if lang_module == 'requests.get':
            try:
                import requests
                requests.get = func
            except ImportError:
                pass

        if lang_module == 'urllib.request.urlopen':
            try:
                import urllib.request
                urllib.request.urlopen = func
            except ImportError:
                pass
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error setting wrapper', error_data)


# Wrapper
def needle_django_get_response(*args, **kwargs):
    lang_module = 'django.core.handlers.base.BaseHandler.get_response'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:' + lang_module, error_data)

    if needle_app.debug_mode: print('Needle.sh: Inside Django get_response')

    try:
        needle_app.total_requests += 1

        try:
            request_data = RequestData()
            request_data.req_start_ts = ts_fn_start

            values = []
            for key, value in args[1].GET.items():
                values.append({'type': 'get', 'name': key, 'value': value})

            for key, value in args[1].POST.items():
                values.append({'type': 'post', 'name': key, 'value': value})

            path_args = args[1].path.split('/')
            for p in path_args:
                if p == '': continue
                values.append({'type': 'path', 'name': 'path', 'value': p})

            request_data.data = values

            request_data.remote_addr = args[1].META['REMOTE_ADDR']
            request_data.request_method = args[1].META['REQUEST_METHOD']
            request_data.http_host = args[1].META['HTTP_HOST']
            request_data.path_info = args[1].META['PATH_INFO']
            request_data.http_user_agent = args[1].META['HTTP_USER_AGENT']

            needle_data.req_data = request_data
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error adding request data to thread storage:', error_data)

        # Check for security scanner
        block_request = False
        try:
            sec_module = 'scan'
            scan_check, action = needle_app.module_active(sec_module)
            if scan_check:
                needle_app.inc_mod_requests(sec_module, lang_module)
                match, arg_type, arg_name, arg_value = check_sec_scanner()
                if match:
                    if needle_app.debug_mode: print('Needle.sh: New Incident of type: Security scanner')

                    if action == 'block': block_request = True

                    # Save request action to thread-data
                    needle_data.req_data.incident_action = action
                    needle_data.req_data.incident_module = sec_module

                    # Add mal request
                    needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)

        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error checking security scanner:', error_data)

        if block_request:
            return ''

        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, '_basic', lang_module, fn_time)

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error while adding request data to thread storage', error_data)

    # Call original method
    response = get_orig(lang_module)(*args, **kwargs)

    try:
        # Insert security related HTTP headers
        sec_headers = needle_app.get_sec_headers()
        for i, (key, value) in enumerate(sec_headers.items()):
            response[key] = value

        if len(sec_headers) > 0: needle_app.inc_mod_requests('add_headers', lang_module)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error while adding security headers:'+lang_module, error_data)

    try:
        req_time = int(time.time() * 1000) - needle_data.req_data.req_start_ts
        needle_app.add_perf_time(needle_data.req_data, '_req', lang_module, req_time)

        needle_app.add_perf_record(needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return response


# Save Request Get/Post params
def needle_flask_request_started(*args, **kwargs):
    lang_module = 'flask.signals.request_started'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:' + lang_module, error_data)

    try:
        needle_app.total_requests += 1

        from flask import request

        request_data = RequestData()
        request_data.req_start_ts = ts_fn_start

        values = []
        get_params = request.args
        for key, value in get_params.items():
            values.append({'type': 'get', 'name': key, 'value': value})

        post_params = request.form
        for key, value in post_params.items():
            values.append({'type': 'post', 'name': key, 'value': value})

        path_args = request.path.split('/')
        for p in path_args:
            if p == '': continue
            values.append({'type': 'path', 'name': 'path', 'value': p})

        request_data.data = values

        request_data.remote_addr = request.remote_addr
        request_data.request_method = request.method
        request_data.http_host = request.host
        request_data.path_info = request.path
        request_data.http_user_agent = request.user_agent.string

        needle_data.req_data = request_data
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error while adding request data to thread storage', error_data)

    block_request = False
    try:
        block_request = needle_scan_module(lang_module)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error while checking sec scanner', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, '_basic', lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)


def needle_flask_request_finished(*args, **kwargs):
    lang_module = 'flask.signals.request_finished'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:' + lang_module, error_data)

    response = kwargs['response']

    try:
        # Insert security related HTTP headers
        sec_headers = needle_app.get_sec_headers()
        for i, (key, value) in enumerate(sec_headers.items()):
            response.headers[key] = value

        if len(sec_headers) > 0: needle_app.inc_mod_requests('add_headers', lang_module)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, '_basic', lang_module, fn_time)

        req_time = int(time.time() * 1000) - needle_data.req_data.req_start_ts
        needle_app.add_perf_time(needle_data.req_data, '_req', lang_module, req_time)

        needle_app.add_perf_record(needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)


# Check for security scanner
def needle_scan_module(py_module):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    sec_module = 'scan'
    block_request = False

    try:
        scan_check, action = needle_app.module_active(sec_module)
        if scan_check:
            needle_app.inc_mod_requests(sec_module, py_module)
            match, arg_type, arg_name, arg_value = check_sec_scanner()
            if match:

                if needle_app.debug_mode: print('Needle.sh: New Incident of type: Security scanner')

                if action == 'block': block_request = True

                # Save request action to thread-data
                needle_data.req_data.incident_action = action
                needle_data.req_data.incident_module = sec_module

                # Add mal request
                needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking security scanner', error_data)

    return block_request


# Check for XSS attack
def check_content_xss(content):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    try:
        libinjec = needle_app.get_libinjec()

        for obj in needle_data.req_data.data:
            value = obj['value']
            if value == '': continue

            if libinjec:
                resp = libinjec.xss(value)
                if resp == 1:
                    # Check if content contains arg value
                    if content.find(value) > -1:
                        match = True
                        arg_type = obj['type']
                        arg_name = obj['name']
                        arg_value = obj['value']

                        return match, arg_type, arg_name, arg_value

            # If XSS not detected using libinjec module, use regex
            if needle_app.debug_mode: print('Checking XSS using regex...')
            xss_pattern = needle_app.get_xss_pattern()

            if xss_pattern:
                if len(xss_pattern.findall(value)) > 0:
                    # Check if content contains arg value
                    if content.find(value) > -1:
                        match = True
                        arg_type = obj['type']
                        arg_name = obj['name']
                        arg_value = obj['value']

                        return match, arg_type, arg_name, arg_value
            else:
                needle_app.add_error('Error checking XSS:', 'XSS pattern unavailable')
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking XSS:', error_data)

    return match, arg_type, arg_name, arg_value


def needle_django_template_render(*args, **kwargs):
    sec_module = 'xss'
    lang_module = 'django.template.loader.render_to_string'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:'+lang_module, error_data)

    content = ''
    try:
        # Add module used
        needle_app.add_module(sec_module, 'django.template.loader', 'render_to_string')

        # Get HTML content
        content = get_orig(lang_module)(*args, **kwargs)

        # Save request action to thread-data
        if needle_data.req_data.incident_action == 'block':
            content = needle_app.get_blocked_page_content(needle_data.req_data.incident_module)
        else:
            # Check if XSS module is active
            xss_check, action = needle_app.module_active(sec_module)
            if xss_check:
                needle_app.inc_mod_requests(sec_module, lang_module)
                if needle_app.debug_mode: print('Needle.sh: Checking XSS')

                match, arg_type, arg_name, arg_value = check_content_xss(content)
                if match:
                    if needle_app.debug_mode: print('Needle.sh: New Incident of type: XSS')

                    if action == 'block':
                        # Show blocked message
                        content = needle_app.get_blocked_page_content('xss')

                    # Add mal request
                    needle_app.add_mal_request(action, 'xss', arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking reflected XSS:', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return content


# Monkey patch: Render template function
def needle_flask_render_template(*args, **kwargs):
    sec_module = 'xss'
    lang_module = 'flask.render_template'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:'+lang_module, error_data)

    try:
        needle_app.add_module(sec_module, 'flask', lang_module)
    except Exception as e:
        pass

    content = ''
    try:
        # Get HTML content
        content = get_orig(lang_module)(*args, **kwargs)

        # Save request action to thread-data
        if needle_data.req_data.incident_action == 'block':
            content = needle_app.get_blocked_page_content(needle_data.req_data.incident_module)
        else:
            # Check if XSS module is active
            xss_check, action = needle_app.module_active(sec_module)
            if xss_check:
                needle_app.inc_mod_requests(sec_module, lang_module)
                if needle_app.debug_mode: print('Checking XSS...')

                match, arg_type, arg_name, arg_value = check_content_xss(content)
                if match:
                    if needle_app.debug_mode: print('Needle.sh: New Incident of type: XSS')

                    if action == 'block':
                        # Show blocked message
                        content = needle_app.get_blocked_page_content('xss')

                    # Add mal request
                    needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking reflected XSS', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return content


# Check command injection
def check_command_injection(command):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    try:
        cmdi_pattern = needle_app.get_cmdi_pattern()

        if cmdi_pattern:
            for obj in needle_data.req_data.data:
                value = obj['value']
                if value == '': continue

                remove_chars = ['\'', '"', '\\', '$@', '`', '$(', ')']  # Remove characters that will be ignored by command shell
                for c in remove_chars:
                    value = value.replace(c, '')
                    command = command.replace(c, '')

                if value == '': continue

                if len(cmdi_pattern.findall(value)) > 0:
                    # Check if command contains arg value
                    if command.find(value) > -1:
                        match = True
                        arg_type = obj['type']
                        arg_name = obj['name']
                        arg_value = obj['value']
                        return match, arg_type, arg_name, arg_value
        else:
            needle_app.add_error('Error checking command injection:', 'Unavailable cmdi pattern')

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking command injection', error_data)

    return match, arg_type, arg_name, arg_value


# Check command injection
def needle_cmdi_check(lang_module, *args, **kwargs):
    sec_module = 'cmdi'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:' + lang_module, error_data)

    try:
        # Add module used
        needle_app.add_module(sec_module, '', lang_module)
    except Exception as e:
        pass

    try:
        cmdi_check, action = needle_app.module_active(sec_module)

        if cmdi_check:
            needle_app.inc_mod_requests(sec_module, lang_module)
            match, arg_type, arg_name, arg_value = check_command_injection(args[0])

            if match:
                if needle_app.debug_mode: print('Needle.sh: New Incident of type: Command injection')

                if action == 'block':
                    # Replace with blank command
                    args = ('',)

                    # Save request action to thread-data
                    needle_data.req_data.incident_action = 'block'
                    needle_data.req_data.incident_module = sec_module

                # Add mal request
                needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking command injection', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    # Call original function
    return get_orig(lang_module)(*args, **kwargs)


# Instrumented method for os.system
def needle_os_system(*args, **kwargs):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()
    try:
        py_module = 'os.system'
        return needle_cmdi_check(py_module, *args, **kwargs)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking command injection', error_data)


# Instrumented method for os.popen
def needle_os_popen(*args, **kwargs):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()
    try:
        py_module = 'os.popen'
        return needle_cmdi_check(py_module, *args, **kwargs)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking command injection', error_data)


# Check LFI
def check_lfi(filepath):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    try:
        for obj in needle_data.req_data.data:
            value = obj['value']
            if value == '': continue

            if filepath == value and (filepath[0] == '/' or filepath.find('../') > -1):
                match = True
                arg_type = obj['type']
                arg_name = obj['name']
                arg_value = obj['value']
                return match, arg_type, arg_name, arg_value

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking LFI', error_data)

    return match, arg_type, arg_name, arg_value


# Check LFI
def needle_lfi_check(lang_module, *args, **kwargs):
    sec_module = 'lfi'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:'+lang_module, error_data)

    try:
        needle_app.add_module(sec_module, '', lang_module)
    except Exception as e:
        pass

    try:
        lfi_check, action = needle_app.module_active(sec_module)

        if lfi_check:
            needle_app.inc_mod_requests(sec_module, lang_module)
            match, arg_type, arg_name, arg_value = check_lfi(args[0])

            if match:
                if needle_app.debug_mode: print('Needle.sh: New Incident of type: Local File Inclusion')

                if action == 'block':
                    # Replace with blank
                    args = ('',)

                    # Save request action to thread-data
                    needle_data.req_data.incident_action = 'block'
                    needle_data.req_data.incident_module = sec_module

                # Add mal request
                needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking LFI', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return get_orig(lang_module)(*args, **kwargs)


def needle_builtins_open(*args, **kwargs):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    #print('Needle.sh: Inside builtins.open:', args)

    py_module = 'builtins.open'
    return needle_lfi_check(py_module, *args, **kwargs)


def check_private_ip(host_string):
    prefixes = [
        "127.", "0.",
        "10.", "172.16.", "172.17.", "172.18.", "172.19.", "172.20.", "172.21.", "172.22.", "172.23.", "172.24.",
        "172.25.", "172.26.", "172.27.", "172.28.", "172.29.", "172.30.", "172.31.", "192.168.", "169.254.",
        "fc", "fe", "ff", "::1", "localhost"
    ]

    # Check if host is IP address or number
    parts = host_string.split(".")
    mod_parts = []

    for p in parts:
        mod_p = p
        is_int = False

        # If hex, convert to dec
        if p.startswith('0x') or p.startswith('0X'):
            #s = p[2:len(p)]
            try:
                if hex(int(p, 16)) == p.lower():
                    is_int = True
                    mod_p = str(int(p, 16))
            except Exception as e:
                pass

        elif p.startswith('0'): #If octal, convert to dec
            s = p.lstrip('0')
            try:
                if str(int(s)) == s:
                    is_int = True
                    mod_p = str(int(s, 8))
            except Exception as e:
                pass

        try:
            if str(int(p)) == p:
                is_int = True
        except Exception as e:
            pass

        if len(parts) == 1 and is_int:
            try:
                import socket, struct
                mod_p = socket.inet_ntoa(struct.pack('!L', int(mod_p)))
            except Exception as e:
                pass

        mod_parts.append(mod_p)

    mod_host_string = '.'.join(mod_parts)
    is_private = False

    # str_ip_addr = ''
    # try:
    #     int_url_host = int(url_host)
    #     if str(int_url_host) == url_host:
    #         import socket, struct
    #         str_ip_addr = socket.inet_ntoa(struct.pack('!L', int_url_host))
    # except Exception as e:
    #     pass

    for prefix in prefixes:
        #if url_host.startswith(prefix) or str_ip_addr.startswith(prefix):
        if mod_host_string.startswith(prefix):
            is_private = True
            break

    return is_private


def check_ssrf(url_string):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    try:
        from urllib.parse import urlparse
        u = urlparse(url_string)
        url_host = u.netloc

        for obj in needle_data.req_data.data:
            value = obj['value']
            if value == '': continue

            if url_string == value and check_private_ip(url_host):
                match = True
                arg_type = obj['type']
                arg_name = obj['name']
                arg_value = obj['value']
                return match, arg_type, arg_name, arg_value

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking SSRF', error_data)

    return match, arg_type, arg_name, arg_value


# Check SSRF
def needle_ssrf_check(lang_module, *args, **kwargs):
    sec_module = 'ssrf'

    try:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
    except Exception as e:
        return get_orig(lang_module)(*args, **kwargs)

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:'+lang_module, error_data)

    try:
        needle_app.add_module(sec_module, '', lang_module)
    except Exception as e:
        pass

    try:
        ssrf_check, action = needle_app.module_active(sec_module)

        if ssrf_check:
            needle_app.inc_mod_requests(sec_module, lang_module)
            match, arg_type, arg_name, arg_value = check_ssrf(args[0])

            if match:
                if needle_app.debug_mode: print('Needle.sh: New Incident of type: Server Side Request Forgery (SSRF)')

                if action == 'block':
                    # Replace with blank
                    args = ('',)

                    # Save request action to thread-data
                    needle_data.req_data.incident_action = 'block'
                    needle_data.req_data.incident_module = sec_module

                # Add mal request
                needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking SSRF', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return get_orig(lang_module)(*args, **kwargs)


def needle_requests_get(*args, **kwargs):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    if needle_app.debug_mode: print('Needle.sh: Inside needle_requests_get:', args)

    py_module = 'requests.get'
    return needle_ssrf_check(py_module, *args, **kwargs)


def needle_urllib_request_urlopen(*args, **kwargs):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    if needle_app.debug_mode: print('Needle.sh: Inside needle_urllib_request_urlopen:', args)

    py_module = 'urllib.request.urlopen'
    return needle_ssrf_check(py_module, *args, **kwargs)


# Check SQL injection
def check_sql_injection(query):
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    if needle_app.debug_mode: print('Needle.sh: Checking SQL injection')

    try:
        libinjec = needle_app.get_libinjec()

        for obj in needle_data.req_data.data:
            value = obj['value']

            if value == '': continue

            if libinjec:
                resp = libinjec.sqli(value, '')
                if resp == 1:
                    # Check if SQL query contains arg value
                    if query.find(value) > -1:
                        match = True
                        arg_type = obj['type']
                        arg_name = obj['name']
                        arg_value = obj['value']

                        # Save request action to thread-data
                        needle_data.req_data.incident_action = 'block'
                        needle_data.req_data.incident_module = 'sqli'
                        needle_data.req_data.incident_rule = 'sqli_1'

                        return match, arg_type, arg_name, arg_value

            # If no sql injection detected with libinjection, use regex
            pattern = needle_app.get_sqli_pattern()

            if len(value.split()) > 1 and len(pattern.findall(value)) > 0 and query.find(value) > -1:
                match = True
                arg_type = obj['type']
                arg_name = obj['name']
                arg_value = obj['value']

                # Save request action to thread-data
                needle_data.req_data.incident_action = 'block'
                needle_data.req_data.incident_module = 'sqli'
                needle_data.req_data.incident_rule = 'sqli_2'

                return match, arg_type, arg_name, arg_value

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking SQL injection:', error_data)

    return match, arg_type, arg_name, arg_value


# Instrumented function for mysql.connection.cursor.execute
def needle_sql_cursor_execute(*args, **kwargs):
    lang_module = 'mysql.connection.cursor.execute'
    sec_module = 'sqli'

    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    ts_fn_start = None
    try:
        ts_fn_start = int(time.time() * 1000)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error starting perf tracking:'+lang_module, error_data)

    try:
        needle_app.add_module(sec_module, 'mysql.connection.cursor', 'execute')
    except Exception as e:
        pass

    try:
        # Check for SQL injection
        sqli_check, action = needle_app.module_active(sec_module)
        if sqli_check:
            needle_app.inc_mod_requests(sec_module, lang_module)
            match, arg_type, arg_name, arg_value = check_sql_injection(args[0])
            if match:
                if needle_app.debug_mode: print('Needle.sh: New Incident of type: SQL injection (rule: '+needle_data.req_data.incident_rule+')')

                if action == 'block':
                    # Change query to blank query
                    args = ('-- Query blocked by Needle.sh agent (Possible SQL injection)',)

                # Add mal request
                needle_app.add_mal_request(action, sec_module, arg_type, arg_name, arg_value, needle_data.req_data)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking SQL injection', error_data)

    try:
        fn_time = int(time.time() * 1000) - ts_fn_start
        needle_app.add_perf_time(needle_data.req_data, sec_module, lang_module, fn_time)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error adding perf time:' + lang_module, error_data)

    return needle_app.orig_sql_cursor_execute(*args, **kwargs)


# Wrapper class around mysql.connection.cursor
class NeedleSqlCursor():
    def __init__(self, cursor):
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
        try:
            self.cursor = cursor
            # self.execute = patcher(self.cursor.execute)
            needle_app.orig_sql_cursor_execute = self.cursor.execute
            self.execute = needle_sql_cursor_execute
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error initialising cursor object:', error_data)

    def __getattr__(self, name):
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
        try:
            return getattr(self.cursor, name)
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error returning custom cursor method:', error_data)


# Wrapper class around mysql.connection
class NeedleSqlConnection():
    def __init__(self, connection):
        from .needle_app import get_needle_app
        needle_app = get_needle_app()

        try:
            self.connection = connection
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error initialising custom SQL connection object:', error_data)

    def cursor(self, *args, **kwargs):
        from .needle_app import get_needle_app
        needle_app = get_needle_app()

        try:
            orig_cursor = self.connection.cursor(*args, **kwargs)
            return NeedleSqlCursor(orig_cursor)
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error getting cursor object from SQL connection:', error_data)

    # Handle unknown method calls
    def __getattr__(self, name):
        from .needle_app import get_needle_app
        needle_app = get_needle_app()

        try:
            return getattr(self.connection, name)
        except Exception as e:
            error_data = str(e)
            needle_app.add_error('Error returning custom connection method:', error_data)


# Instrumented function for MySQL connect
def needle_mysql_connect(*args, **kwargs):
    py_module = 'mysql.connector.connect'

    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    try:
        conn = get_orig(py_module)(*args, **kwargs)
        return NeedleSqlConnection(conn)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error in instrumented MySQL connect:', error_data)


# Instrumented function for MySQL connect
def needle_psycopg2_connect(*args, **kwargs):
    py_module = 'psycopg2.connect'

    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    try:
        # conn = needle_app.orig_mysql_connect(*args, **kwargs)
        conn = get_orig(py_module)(*args, **kwargs)
        return NeedleSqlConnection(conn)
    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error in instrumented psycopg2 connect:', error_data)


# Check security scanner
def check_sec_scanner():
    from .needle_app import get_needle_app
    needle_app = get_needle_app()

    match = False
    arg_type = ''
    arg_name = ''
    arg_value = ''

    if needle_app.debug_mode: print('Needle.sh: Checking security scanner')

    try:
        scan_pattern = needle_app.get_scanner_pattern()
        if not scan_pattern: return match, arg_type, arg_name, arg_value

        value = needle_data.req_data.http_user_agent

        if value == '': return match, arg_type, arg_name, arg_value

        if len(scan_pattern.findall(value)) > 0:
            match = True
            arg_type = 'http_header'
            arg_name = 'user_agent'
            arg_value = value

            return match, arg_type, arg_name, arg_value

    except Exception as e:
        error_data = str(e)
        needle_app.add_error('Error checking security scanner:', error_data)

    return match, arg_type, arg_name, arg_value



import requests
import json
import time
import platform
import re
import time

# Class to pass request data
class RequestData:
    data = []
    remote_addr = ''
    request_method = ''
    http_host = ''
    path_info = ''
    http_user_agent = ''
    incident_action = ''
    incident_module = ''
    incident_rule = ''
    req_start_ts = None
    perf_data = []

    def __init__(self):
        self.perf_data = []


# Class to keep track of instrumented methods
class InstrMethod:
    sec_module = ''
    py_module = ''
    orig_method = None
    is_instr = False


class PerfTime:
    sec_module = ''
    lang_module = ''
    time = 0

    def __init__(self, sec_module, lang_module, time):
        self.sec_module = sec_module
        self.lang_module = lang_module
        self.time = time

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__)


class PerfRecord:
    path = ''
    req_ts = 0
    perf_times = []

    def __init__(self, path, req_ts, perf_times):
        self.path = path
        self.req_ts = req_ts
        self.perf_times = perf_times

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__)


class NeedleApp:
    agent_version = ''
    app_id = ''
    api_key = ''
    server_url = ''
    platform = 'python'
    framework = ''
    project_dir = ''
    settings = {}
    app_active = False
    errors = []
    total_requests = 0
    mal_requests = []
    module_requests = []
    modules_used = []
    test_mode = False
    debug_mode = False
    instr_list = []
    is_instr = False
    libinjec = None
    sqli_pattern = None
    xss_pattern = None
    cmdi_pattern = None
    mdbi_pattern = None
    scan_pattern = None
    orig_sql_cursor_execute = None
    show_blocked_message = False
    use_libinjec = True # Mark false if libinjection should not be used
    flask_app = None
    run_agent = False
    perf_data = []
    perf_active = True
    perf_collect = True
    perf_count = 0
    perf_max = 100
    perf_interval = 12
    perf_collect_ts = 0.0

    def __init__(self, debug=False, show_blocked_message=False, flask_app=None):
        self.debug_mode = debug
        self.show_blocked_message = show_blocked_message
        self.flask_app = flask_app

        # Check if framework supported
        if not self.detect_framework():
            # Framework not supported, stop execution
            print("Needle.sh error: Web framework not supported")
            return

        # Load settings from needle.json
        try:
            filepath = self.project_dir + '/needle.json'
            with open(filepath) as fp:
                content = fp.read()
                needle_settings = json.loads(content)
                self.app_id = needle_settings['app_id']
                self.api_key = needle_settings['api_key']
                self.server_url = needle_settings['server_url']
                test_mode = needle_settings['test_mode']
                self.test_mode = False
                if test_mode == 1:
                    self.test_mode = True
                    print('Needle.sh: Agent in Test Mode')
        except Exception as e:
            error_data = str(e)
            self.add_error(
                'Error opening needle.json file. Please make sure that the file is present in your project\'s '
                'root folder', error_data)
            self.add_error('Error opening needle.json. Trying needle.ini', '')

        if self.app_id == '' or self.api_key == '':
            # Load settings from .ini file
            try:
                filepath = self.project_dir + '/needle.ini'
                with open(filepath) as fp:
                    for cnt, line in enumerate(fp):
                        # print("line = " + line)
                        if line[0] != '#':
                            s_name, s_value = line.strip().split('=')
                            if s_name == 'app_id': self.app_id = s_value
                            if s_name == 'api_key': self.api_key = s_value
                            if s_name == 'server_url': self.server_url = s_value
                            if s_name == 'test_mode':
                                if s_value == '0':
                                    self.test_mode = False
                                elif s_value == '1':
                                    self.test_mode = True
                                    print('Needle.sh: Agent in Test Mode')
            except Exception as e:
                error_data = str(e)
                self.add_error('Error opening needle.ini file. Please make sure that the file is present in your project\'s '
                               'root folder', error_data)
            return

        if self.app_id == '' or self.api_key == '':
            print("Needle.sh error: App ID or API key incorrect")
        else:
            self.run_agent = True

        return

    # Detect project framework
    def detect_framework(self):
        detected = False
        # Check for Django framework
        try:
            from django.conf import settings
            self.framework = 'django'
            self.project_dir = settings.BASE_DIR
            detected = True
        except Exception as e:
            pass

        try:
            import flask
            self.framework = 'flask'
            self.project_dir = self.flask_app.root_path
            detected = True
        except Exception as e:
            pass

        return detected

    # Add error
    def add_error(self, error, error_data):
        if self.debug_mode: print('Needle.sh: Error! ', error, error_data)
        self.errors.append({'platform': self.platform, 'error': error, 'error_data': error_data})

    # Add malicious request
    def add_mal_request(self, action, reason, arg_type, arg_name, arg_value, req_data):
        from .utilities import clean_server_values

        # Check for sensitive data
        arg_name, arg_value = clean_server_values(arg_name, arg_value)

        mal_req = {}
        mal_req['type'] = action
        mal_req['reason'] = reason
        mal_req['arg_type'] = arg_type
        mal_req['arg_name'] = arg_name
        mal_req['arg_value'] = arg_value
        mal_req['client_ip'] = req_data.remote_addr
        mal_req['http_method'] = req_data.request_method
        mal_req['server'] = req_data.http_host
        mal_req['path'] = req_data.path_info
        mal_req['user_agent'] = req_data.http_user_agent

        if self.debug_mode: print('Needle.sh: Adding incident: ', mal_req)
        self.mal_requests.append(mal_req)

    # Increment module request count
    def inc_mod_requests(self, sec_module, lang_module):
        index = -1

        for i, obj in enumerate(self.module_requests):
            if obj['sec_module'] == sec_module and obj['lang_module'] == lang_module:
                index = i
                break

        if index == -1:
            self.module_requests.append({'sec_module': sec_module, 'lang_module': lang_module, 'count': 1})
        else:
            self.module_requests[index]['count'] += 1

    # Add module
    def add_module(self, type, package, method):
        module = {'type': type, 'package': package, 'method': method}
        if not (module in self.modules_used): self.modules_used.append(module)

    # Call APIs at regular intervals
    def api_thread(self):
        settings_ping_count = 0
        errors_count = 0
        app_info_count = 0

        try:
            while True:
                # Get app settings
                if settings_ping_count == 0:
                    self.api_get_settings()

                # Send total_req data every minute
                if self.app_active and (self.total_requests > 0 or len(self.mal_requests) > 0):
                    self.api_send_req_data()

                # Send agent errors
                if self.app_active and errors_count == 0 and (len(self.errors) > 0):
                    self.api_send_app_info('errors')

                # Send modules used data
                if self.app_active and app_info_count == 0:
                    self.api_send_app_info('app_info')

                settings_ping_count += 1
                errors_count += 1
                app_info_count += 1

                if settings_ping_count == 1: settings_ping_count = 0
                if errors_count == 1: errors_count = 0
                if app_info_count == 1: app_info_count = 0  # Send frequency = 10 mins

                # Pause for 60 seconds
                time.sleep(60)
        except Exception as e:
            error_data = str(e)
            self.add_error('Error while sending req data', error_data)

    # Get payload for API calls
    def get_api_payload(self):
        test_mode = 0
        if self.test_mode: test_mode = 1
        libinjec = 0
        if self.get_libinjec(): libinjec = 1

        payload = {'app_id': self.app_id, 'api_key': self.api_key, 'test_mode': test_mode, 'libinjec': libinjec,
                   'platform': self.platform, 'framework': self.framework, 'agent_version': self.agent_version}
        return payload

    # Send app info - agent errors, modules used
    def api_send_app_info(self, info):
        if self.debug_mode: print('Needle.sh: Sending app info data')

        try:
            url = self.server_url + '/api/store_app_info'
            data = self.get_api_payload()

            if info == 'errors' and len(self.errors) > 0:
                data['agent_errors'] = self.errors
                self.errors = []

            if info == 'app_info':
                if len(self.modules_used) > 0:
                    data['modules_used'] = self.modules_used
                    self.modules_used = []

                if len(self.perf_data) > 0:
                    data['perf_data'] = self.perf_data
                    self.perf_data = []

            json_data = json.dumps(data)
            x = requests.post(url, data=json_data)
        except Exception as e:
            error_data = str(e)
            self.add_error('Error while sending app info', error_data)

    # Send requests data - total requests, malicious requests
    def api_send_req_data(self):
        if self.debug_mode: print('Needle.sh: Sending requests data')

        data = {}
        try:
            url = self.server_url + '/api/store_requests'
            data = self.get_api_payload()

            if self.total_requests > 0:
                data['total_requests'] = self.total_requests
                self.total_requests = 0

            if len(self.mal_requests) > 0:
                data['mal_requests'] = self.mal_requests
                self.mal_requests = []

            if len(self.module_requests) > 0:
                data['mod_requests'] = self.module_requests
                self.module_requests = []

            json_data = json.dumps(data)
            req = requests.post(url, data=json_data)
        except Exception as e:
            error_data = str(e)
            self.add_error('Error while sending req data', error_data)
            # Since requests data could not be sent, save in variable
            # Todo: Make sure that total_req and mal_req dont become too big in size
            self.total_requests += data['total_requests']
            if len(data['mal_requests']) > 0:
                a = data['mal_requests']
                b = self.mal_requests
                a.extend(b)
                self.mal_requests = a

            if len(data['mod_requests']) > 0:
                a = data['mod_requests']
                b = self.module_requests
                a.extend(b)
                self.module_requests = a

    # Get app settings
    def api_get_settings(self):
        if self.debug_mode: print('Needle.sh: Getting app settings')

        try:
            api_url = self.server_url + '/api/get_app_settings'
            # if self.debug_mode: print('Needle API: get app settings. url = ', api_url)
            data = self.get_api_payload()
            json_data = json.dumps(data)

            req = requests.post(api_url, data=json_data)
            if self.debug_mode: print("Needle.sh: Received app settings = ", req.text)

            resp = json.loads(req.text)
            self.settings = resp['settings']
        except Exception as e:
            error_data = str(e)
            self.add_error('Error while fetching settings', error_data)

        try:
            if self.settings['active'] == 1:
                self.app_active = True
                self.instrument('basic', True)

                if 'sqli' in self.settings and self.settings['sqli']['active'] == 1:
                    self.instrument('sqli', True)
                else:
                    self.instrument('sqli', False)

                if 'xss' in self.settings and self.settings['xss']['active'] == 1:
                    self.instrument('xss', True)
                else:
                    self.instrument('xss', False)

                if 'cmdi' in self.settings and self.settings['cmdi']['active'] == 1:
                    self.instrument('cmdi', True)
                else:
                    self.instrument('cmdi', False)

                if 'lfi' in self.settings and self.settings['lfi']['active'] == 1:
                    self.instrument('lfi', True)
                else:
                    self.instrument('lfi', False)

                if 'ssrf' in self.settings and self.settings['ssrf']['active'] == 1:
                    self.instrument('ssrf', True)
                else:
                    self.instrument('ssrf', False)

            elif self.settings['active'] == 0:
                self.app_active = False
                self.instrument('basic', False)
                self.instrument('sqli', False)
                self.instrument('xss', False)
                self.instrument('cmdi', False)
                self.instrument('lfi', False)
                self.instrument('ssrf', False)
        except Exception as e:
            error_data = str(e)
            self.add_error('Error while instrumenting', error_data)

    # Update instr status for module
    def update_instr_status(self, sec_module, py_module, orig_method, is_instr):
        is_present = False
        for index, obj in enumerate(self.instr_list):
            if obj.sec_module == sec_module and obj.py_module == py_module:
                is_present = True
                self.instr_list[index].is_instr = is_instr
                break

        # If not present, add object with new status
        if not is_present:
            instr_method = InstrMethod()
            instr_method.sec_module = sec_module
            instr_method.py_module = py_module
            instr_method.orig_method = orig_method
            instr_method.is_instr = is_instr
            self.instr_list.append(instr_method)

        return is_instr

    # Get instrumentation status for module
    def get_module_status(self, py_module):
        is_instr = False
        for obj in self.instr_list:
            if obj.py_module == py_module and obj.is_instr:
                is_instr = True
                break

        return is_instr

    # Return original method for instrumented method
    def get_orig_method(self, py_module):
        for obj in self.instr_list:
            if obj.py_module == py_module:
                return obj.orig_method

    # Instrument
    def instrument(self, sec_module, is_instr):
        from .wrappers import set_wrapper, get_wrapper
        instr_module_list = [
            {'sec_module': 'basic', 'framework': 'django',
             'py_module': 'django.core.handlers.base.BaseHandler.get_response'},
            {'sec_module': 'basic', 'framework': 'flask', 'py_module': 'flask.signals.request_started'},
            {'sec_module': 'basic', 'framework': 'flask', 'py_module': 'flask.signals.request_finished'},
            {'sec_module': 'xss', 'framework': 'django', 'py_module': 'django.template.loader.render_to_string'},
            {'sec_module': 'xss', 'framework': 'flask', 'py_module': 'flask.render_template'},
            {'sec_module': 'sqli', 'framework': '', 'py_module': 'mysql.connector.connect'},
            {'sec_module': 'sqli', 'framework': '', 'py_module': 'psycopg2.connect'},
            {'sec_module': 'cmdi', 'framework': '', 'py_module': 'os.system'},
            {'sec_module': 'cmdi', 'framework': '', 'py_module': 'os.popen'},
            {'sec_module': 'lfi', 'framework': '', 'py_module': 'builtins.open'},
            {'sec_module': 'ssrf', 'framework': '', 'py_module': 'requests.get'},
            {'sec_module': 'ssrf', 'framework': '', 'py_module': 'urllib.request.urlopen'}
        ]

        for mod in instr_module_list:
            if mod['sec_module'] != sec_module: continue
            if mod['framework'] != '' and mod['framework'] != self.framework: continue

            py_module = mod['py_module']

            if is_instr != self.get_module_status(py_module):
                if is_instr:
                    # Instrument module
                    if self.debug_mode: print('Needle.sh: Instrumenting module:', py_module)
                    try:
                        if py_module == 'flask.signals.request_started':
                            try:
                                import flask
                                flask.request_started.connect(get_wrapper(py_module), self.flask_app)
                                continue
                            except ImportError:
                                pass
                        elif py_module == 'flask.signals.request_finished':
                            try:
                                import flask
                                flask.request_finished.connect(get_wrapper(py_module), self.flask_app)
                                continue
                            except ImportError:
                                pass
                        else:
                            set_wrapper(py_module, True)

                        # Update instr status for py_module
                        self.update_instr_status(sec_module, py_module, None, True)
                    except Exception as e:
                        error_data = str(e)
                        self.add_error('Error while instrumenting module: ' + py_module, error_data)
                else:
                    # Uninstrument module
                    if self.debug_mode: print('Un-instrumenting module:', py_module)
                    try:
                        if py_module == 'flask.signals.request_started':
                            try:
                                import flask, blinker
                                flask.request_started.disconnect(get_wrapper(py_module), sender=blinker.base.ANY)
                                continue
                            except ImportError:
                                pass
                        elif py_module == 'flask.signals.request_finished':
                            try:
                                import flask, blinker
                                flask.request_finished.disconnect(get_wrapper(py_module), sender=blinker.base.ANY)
                                continue
                            except ImportError:
                                pass
                        else:
                            set_wrapper(py_module, False)

                        # Update instr status for py_module
                        self.update_instr_status(sec_module, py_module, None, False)
                    except Exception as e:
                        error_data = str(e)
                        self.add_error('Error while un-instrumenting module: ' + py_module, error_data)

    # Get security headers to be inserted
    def get_sec_headers(self):
        headers = {}

        try:
            if self.app_active:
                if 'headers' in self.settings:
                    headers = self.settings['headers']
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting security headers: ', error_data)

        return headers

    # Check if security module active
    def module_active(self, sec_module):
        active = False
        action = ''

        try:
            if self.app_active and sec_module in self.settings and self.settings[sec_module]['active'] == 1:
                active = True
                action = self.settings[sec_module]['action']
        except Exception as e:
            error_data = str(e)
            self.add_error('Error checking module active - '+sec_module+':', error_data)

        return active, action

    # Get libinjection module
    def get_libinjec(self):

        # Check the use_libinjec setting
        if not self.use_libinjec:
            return None

        try:
            if self.libinjec:
                return self.libinjec
            if not self.libinjec:
                libinjec = None
                op_system = platform.system()
                if op_system == 'Darwin':  # Mac OS
                    from .libinjection2.mac_x86_64 import libinjection
                    libinjec = libinjection
                elif op_system == 'Linux':
                    from .libinjection2.linux import libinjection
                    libinjec = libinjection
                elif op_system == '':
                    self.add_error('Error getting libinjec module for platform: ', 'Unrecognised platform')

                self.libinjec = libinjec
                return self.libinjec
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting libinjec module for platform: ', error_data)
            return None

    # Get XSS pattern
    def get_xss_pattern(self):
        try:
            if self.xss_pattern:
                return self.xss_pattern
            else:
                import os, sys, inspect
                current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
                filepath = current_dir + '/data/js_event'

                str_pattern = ''
                with open(filepath) as fp:
                    for cnt, line in enumerate(fp):
                        line = line.strip()
                        if line == '' or line[0] == '#': continue

                        str_pattern += line + '|'

                str_pattern = str_pattern.rstrip('|')
                str_pattern = r'\b(' + str_pattern + r')\b'
                str_pattern = r'(<[\\s]*script[\\s]*[>]*|javascript:|javascript&colon;|FSCommand)|' + str_pattern
                pattern = re.compile(str_pattern, re.IGNORECASE)

                self.xss_pattern = pattern
                return self.xss_pattern
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting XSS pattern:', error_data)
            return None

    # Get sqli pattern
    def get_sqli_pattern(self):
        try:
            if self.sqli_pattern:
                return self.sqli_pattern
            else:
                pattern = re.compile(
                    r'\b(select|update|insert|alter|create|drop|delete|merge|union|show|exec|or|and|order|sleep|having|'
                    r'xor|like|regexp)\b|(&&|\|\|)',
                    re.IGNORECASE)

                self.sqli_pattern = pattern
                return self.sqli_pattern
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting sqli pattern:', error_data)
            return None

    # Get command injection pattern
    def get_cmdi_pattern(self):
        try:
            if self.cmdi_pattern:
                return self.cmdi_pattern
            else:
                import os, sys, inspect
                current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
                filepath = current_dir + '/data/unix_cmd'

                str_pattern = ''
                with open(filepath) as fp:
                    for cnt, line in enumerate(fp):
                        line = line.strip()
                        if line == '' or line[0] == '#': continue

                        str_pattern += line.rstrip('+') + '|'

                str_pattern = str_pattern.rstrip('|')
                str_pattern = r'(?:;|\{|\||\|\||&|&&|\n|\r|\$\(|\$\(\(|`|\${|<\(|>\(|\(\s*\))\s*' \
                              r'(?:{|\s*\(\s*|\w+=(?:[^\s]*|\$.*|\$.*|<.*|>.*|\'.*\'|\".*\")\s+|!\s*|\$)*\s*(?:\'|\")*' \
                              r'(?:[\?\*\[\]\(\)\-\|+\w\'\"\./\\\\]+\/)?[\\\\\'\"]*'+str_pattern+'\b'

                pattern = re.compile(str_pattern, re.IGNORECASE)

                self.cmdi_pattern = pattern
                return self.cmdi_pattern
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting cmdi pattern:', error_data)
            return None

    # Get security scanner pattern
    def get_scanner_pattern(self):
        try:
            if self.scan_pattern:
                return self.scan_pattern
            else:
                import os, sys, inspect
                current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
                filepath = current_dir + '/data/scan'

                str_pattern = ''
                with open(filepath) as fp:
                    for cnt, line in enumerate(fp):
                        line = line.strip()
                        if line == '' or line[0] == '#': continue

                        str_pattern += line.rstrip('+') + '|'

                str_pattern = str_pattern.rstrip('|')
                str_pattern = r'\b(' + str_pattern + r')\b'

                pattern = re.compile(str_pattern, re.IGNORECASE)

                self.scan_pattern = pattern
                return self.scan_pattern
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting scan pattern:', error_data)
            return None

    # Get project modules
    def get_project_modules(self):
        try:
            import sys
            all_modules = sys.modules.keys()
            module_list = []
            for m in all_modules:
                # Only get parent module name. e.g. If module is mysql.connector, only get mysql
                parts = m.split('.')
                # print(parts[0])
                if parts[0] not in module_list:
                    module_list.append(parts[0])

            # print('Modules = ', len(module_list), module_list)
        except Exception as e:
            error_data = str(e)
            self.add_error('Error getting module list', error_data)

    # Return content for blocked message
    def get_blocked_page_content(self, module_id=''):
        str_content = ''
        if self.show_blocked_message:
            module_name = ''

            if module_id == 'sqli': module_name = 'SQL injection'
            if module_id == 'xss': module_name = 'Cross-site Scripting(XSS)'
            if module_id == 'cmdi': module_name = 'Command injection'
            if module_id == 'scan': module_name = 'Security scanner'
            if module_id == 'lfi': module_name = 'Local File Inclusion (LFI)'
            if module_id == 'ssrf': module_name = 'Server Side Request Forgery (SSRF)'

            str_content = 'Blocked by Needle.sh! Attack type: ' + module_name

        return str_content

    def check_perf_collect(self):
        if not self.perf_active: return False

        if self.perf_collect:
            if self.perf_count > self.perf_max:
                self.perf_collect = False
                self.perf_collect_ts = int(time.time()) + self.perf_interval * 3600
        else:
            # Check if collection should be turned on
            if time.time() > self.perf_collect_ts:
                self.perf_collect = True
                self.perf_count = 0

        return self.perf_collect

    def add_perf_time(self, req_data, sec_module, lang_module, time):
        if self.check_perf_collect():
            #p = PerfTime(sec_module, lang_module, time)
            p = {'sec_module': sec_module, 'lang_module': lang_module, 'time': time}
            req_data.perf_data.append(p)

    def add_perf_record(self, req_data):
        if self.check_perf_collect():
            #p = PerfRecord(req_data.path_info, req_data.req_start_ts, req_data.perf_data)
            perf_times = req_data.perf_data
            p = {'path': req_data.path_info, 'req_ts': req_data.req_start_ts, 'perf_times': perf_times}
            self.perf_data.append(p)
            self.perf_count += 1



# Copyright (c) 2020, Varlogix Technologies
# All rights reserved.
# Our terms: https://needle.sh/terms

from .core.needle_app import init_app


# Start Needle.sh
def start(flask_app=None, debug=False, show_blocked_message=False):
    print("Needle.sh: Starting SDK (version " + sdk_version + ")")

    init_app(debug=debug, show_blocked_message=show_blocked_message, flask_app=flask_app,
             sdk_version=sdk_version)


# SDK version
sdk_version = '1.8.0'



# Check if keys have sensitive info
def clean_server_values(arg_name, arg_value):
    try:
        # Credit-card data in argument value?
        import re
        pattern = r'(\d[ -]*){13,16}'
        pattern = re.compile(pattern, re.IGNORECASE)

        # Sensitive arg names?
        sensitive_arg_names = ['password', 'passwd', 'api_key', 'apikey', 'access_token', 'secret', 'authorization']

        if len(pattern.findall(arg_value)) > 0 or arg_name in sensitive_arg_names:
            # Redact value
            arg_value = '[Sensitive data removed by Needle.sh]'
    except Exception as e:
        from .needle_app import get_needle_app
        needle_app = get_needle_app()
        error_data = str(e)
        needle_app.add_error('Error while checking sensitive data', error_data)

    return arg_name, arg_value


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

packages = ['requests']

setuptools.setup(
    name="needle-sdk",
    version="1.8.0",
    author="Needle.sh team",
    author_email="hello@needle.sh",
    description="Needle.sh SDK for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://needle.sh",
    install_requires=packages,
    packages=['needle_sdk'],
    package_data={
        'needle_sdk': ['core/classes.py', 'core/needle_app.py', 'core/utilities.py', 'core/wrappers.py',
                       'core/libinjection2/linux/_libinjection.so', 'core/libinjection2/linux/libinjection.py',
                       'core/libinjection2/mac_x86_64/_libinjection.so',
                       'core/libinjection2/mac_x86_64/libinjection.py',
                       'core/data/js_event', 'core/data/unix_cmd', 'core/data/scan']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.0',
)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_libinjection')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_libinjection')
    _libinjection = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_libinjection', [dirname(__file__)])
        except ImportError:
            import _libinjection
            return _libinjection
        try:
            _mod = imp.load_module('_libinjection', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _libinjection = swig_import_helper()
    del swig_import_helper
else:
    import _libinjection
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0


def version():
    return _libinjection.version()
version = _libinjection.version

def sqli(s, fingerprint):
    return _libinjection.sqli(s, fingerprint)
sqli = _libinjection.sqli

def xss(s):
    return _libinjection.xss(s)
xss = _libinjection.xss
FLAG_NONE = _libinjection.FLAG_NONE
FLAG_QUOTE_NONE = _libinjection.FLAG_QUOTE_NONE
FLAG_QUOTE_SINGLE = _libinjection.FLAG_QUOTE_SINGLE
FLAG_QUOTE_DOUBLE = _libinjection.FLAG_QUOTE_DOUBLE
FLAG_SQL_ANSI = _libinjection.FLAG_SQL_ANSI
FLAG_SQL_MYSQL = _libinjection.FLAG_SQL_MYSQL
LOOKUP_WORD = _libinjection.LOOKUP_WORD
LOOKUP_TYPE = _libinjection.LOOKUP_TYPE
LOOKUP_OPERATOR = _libinjection.LOOKUP_OPERATOR
LOOKUP_FINGERPRINT = _libinjection.LOOKUP_FINGERPRINT
class sqli_token(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, sqli_token, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, sqli_token, name)
    __repr__ = _swig_repr
    __swig_getmethods__["pos"] = _libinjection.sqli_token_pos_get
    if _newclass:
        pos = _swig_property(_libinjection.sqli_token_pos_get)
    __swig_getmethods__["len"] = _libinjection.sqli_token_len_get
    if _newclass:
        len = _swig_property(_libinjection.sqli_token_len_get)
    __swig_getmethods__["count"] = _libinjection.sqli_token_count_get
    if _newclass:
        count = _swig_property(_libinjection.sqli_token_count_get)
    __swig_getmethods__["type"] = _libinjection.sqli_token_type_get
    if _newclass:
        type = _swig_property(_libinjection.sqli_token_type_get)
    __swig_getmethods__["str_open"] = _libinjection.sqli_token_str_open_get
    if _newclass:
        str_open = _swig_property(_libinjection.sqli_token_str_open_get)
    __swig_getmethods__["str_close"] = _libinjection.sqli_token_str_close_get
    if _newclass:
        str_close = _swig_property(_libinjection.sqli_token_str_close_get)
    __swig_getmethods__["val"] = _libinjection.sqli_token_val_get
    if _newclass:
        val = _swig_property(_libinjection.sqli_token_val_get)

    def __init__(self):
        this = _libinjection.new_sqli_token()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _libinjection.delete_sqli_token
    __del__ = lambda self: None
sqli_token_swigregister = _libinjection.sqli_token_swigregister
sqli_token_swigregister(sqli_token)

class sqli_state(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, sqli_state, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, sqli_state, name)
    __repr__ = _swig_repr
    __swig_getmethods__["s"] = _libinjection.sqli_state_s_get
    if _newclass:
        s = _swig_property(_libinjection.sqli_state_s_get)
    __swig_getmethods__["slen"] = _libinjection.sqli_state_slen_get
    if _newclass:
        slen = _swig_property(_libinjection.sqli_state_slen_get)
    __swig_getmethods__["lookup"] = _libinjection.sqli_state_lookup_get
    if _newclass:
        lookup = _swig_property(_libinjection.sqli_state_lookup_get)
    __swig_getmethods__["userdata"] = _libinjection.sqli_state_userdata_get
    if _newclass:
        userdata = _swig_property(_libinjection.sqli_state_userdata_get)
    __swig_getmethods__["flags"] = _libinjection.sqli_state_flags_get
    if _newclass:
        flags = _swig_property(_libinjection.sqli_state_flags_get)
    __swig_getmethods__["pos"] = _libinjection.sqli_state_pos_get
    if _newclass:
        pos = _swig_property(_libinjection.sqli_state_pos_get)
    __swig_getmethods__["current"] = _libinjection.sqli_state_current_get
    if _newclass:
        current = _swig_property(_libinjection.sqli_state_current_get)
    __swig_getmethods__["fingerprint"] = _libinjection.sqli_state_fingerprint_get
    if _newclass:
        fingerprint = _swig_property(_libinjection.sqli_state_fingerprint_get)
    __swig_getmethods__["reason"] = _libinjection.sqli_state_reason_get
    if _newclass:
        reason = _swig_property(_libinjection.sqli_state_reason_get)
    __swig_getmethods__["stats_comment_ddw"] = _libinjection.sqli_state_stats_comment_ddw_get
    if _newclass:
        stats_comment_ddw = _swig_property(_libinjection.sqli_state_stats_comment_ddw_get)
    __swig_getmethods__["stats_comment_ddx"] = _libinjection.sqli_state_stats_comment_ddx_get
    if _newclass:
        stats_comment_ddx = _swig_property(_libinjection.sqli_state_stats_comment_ddx_get)
    __swig_getmethods__["stats_comment_c"] = _libinjection.sqli_state_stats_comment_c_get
    if _newclass:
        stats_comment_c = _swig_property(_libinjection.sqli_state_stats_comment_c_get)
    __swig_getmethods__["stats_comment_hash"] = _libinjection.sqli_state_stats_comment_hash_get
    if _newclass:
        stats_comment_hash = _swig_property(_libinjection.sqli_state_stats_comment_hash_get)
    __swig_getmethods__["stats_folds"] = _libinjection.sqli_state_stats_folds_get
    if _newclass:
        stats_folds = _swig_property(_libinjection.sqli_state_stats_folds_get)
    __swig_getmethods__["stats_tokens"] = _libinjection.sqli_state_stats_tokens_get
    if _newclass:
        stats_tokens = _swig_property(_libinjection.sqli_state_stats_tokens_get)

    def __init__(self):
        this = _libinjection.new_sqli_state()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _libinjection.delete_sqli_state
    __del__ = lambda self: None
sqli_state_swigregister = _libinjection.sqli_state_swigregister
sqli_state_swigregister(sqli_state)


def sqli_get_token(sqlistate, i):
    return _libinjection.sqli_get_token(sqlistate, i)
sqli_get_token = _libinjection.sqli_get_token

def sqli_init(sql_state, s, flags):
    return _libinjection.sqli_init(sql_state, s, flags)
sqli_init = _libinjection.sqli_init

def is_sqli(sql_state):
    return _libinjection.is_sqli(sql_state)
is_sqli = _libinjection.is_sqli

def sqli_callback(sql_state, fn):
    return _libinjection.sqli_callback(sql_state, fn)
sqli_callback = _libinjection.sqli_callback

def sqli_reset(sql_state, flags):
    return _libinjection.sqli_reset(sql_state, flags)
sqli_reset = _libinjection.sqli_reset

def sqli_fingerprint(sql_state, flags):
    return _libinjection.sqli_fingerprint(sql_state, flags)
sqli_fingerprint = _libinjection.sqli_fingerprint

def sqli_lookup_word(sql_state, lookup_type, s):
    return _libinjection.sqli_lookup_word(sql_state, lookup_type, s)
sqli_lookup_word = _libinjection.sqli_lookup_word

def sqli_tokenize(sql_state):
    return _libinjection.sqli_tokenize(sql_state)
sqli_tokenize = _libinjection.sqli_tokenize

def sqli_fold(sql_state):
    return _libinjection.sqli_fold(sql_state)
sqli_fold = _libinjection.sqli_fold

def sqli_check_fingerprint(sql_state):
    return _libinjection.sqli_check_fingerprint(sql_state)
sqli_check_fingerprint = _libinjection.sqli_check_fingerprint

def sqli_blacklist(sql_state):
    return _libinjection.sqli_blacklist(sql_state)
sqli_blacklist = _libinjection.sqli_blacklist

def sqli_not_whitelist(sql_state):
    return _libinjection.sqli_not_whitelist(sql_state)
sqli_not_whitelist = _libinjection.sqli_not_whitelist
# This file is compatible with both classic and new-style classes.


